#include <mes2.h>
#include "7040s.h"
//#include <h8/reg704x.h>

#define NAK	SDA
#define ACK	0
#define SDA	0x8
#define SCL	0x4
#define DREG	PB.DR.WORD
#define	CREG	PFC.PBIOR.WORD  

//#define DREG	PBDR
//#define	CREG	PBIOR


#define XPA  (*(volatile unsigned short *)0x800000)
#define XPB  (*(volatile unsigned short *)0x800002)
#define XPW  (*(volatile unsigned short *)0x800004)

//T104-DIO72 BASE 0x320
#define DPA1  (*(volatile unsigned char *)0xc00320)
#define DPB1  (*(volatile unsigned char *)0xc00321)
#define DPC1  (*(volatile unsigned char *)0xc00322)
#define DPW1  (*(volatile unsigned char *)0xc00323)

#define DPA2  (*(volatile unsigned char *)0xc00324)
#define DPB2  (*(volatile unsigned char *)0xc00325)
#define DPC2  (*(volatile unsigned char *)0xc00326)
#define DPW2  (*(volatile unsigned char *)0xc00327)

#define DPA3  (*(volatile unsigned char *)0xc00328)
#define DPB3  (*(volatile unsigned char *)0xc00329)
#define DPC3  (*(volatile unsigned char *)0xc0032a)
#define DPW3  (*(volatile unsigned char *)0xc0032b)




#define FT245  (*(volatile unsigned short *)0x80c000)

static void _wait() {
	volatile int w;
	w = 0;
	
//	for(w=0;w<2;w++) ;
}

static void start_i2c() {
	CREG |= (SCL + SDA);
	DREG &= ~SCL;
	DREG |= SDA;
	_wait();
	DREG |= SCL;
	DREG &= ~SDA;
	_wait();
	DREG &= ~SCL;
	CREG &= ~SDA;
}

static void stop_i2c() {
	CREG |= (SCL + SDA);
	DREG &= ~SCL;
	DREG &= ~SDA;
	_wait();
	DREG |= SCL;
	DREG |= SDA;
	_wait();
	DREG &= ~SCL;
	CREG &= ~SDA;
}

static int writebyte(int data) {
	int	i, mask;

	CREG |= (SCL + SDA);
	mask = 0x80;
	for(i = 0;i < 8;i++) {
		if(data & mask) DREG |= SDA;
		else DREG &= ~SDA;
		DREG |= SCL;
		_wait();
		DREG &= ~SCL;
		mask >>= 1;
		_wait();
	}
	CREG &= ~SDA;
	DREG |= SCL;
	_wait();
	i = DREG & SDA;
	DREG &= ~SCL;
	return i;
}

static int readbyte() {
	int	i, data;

	data = 0;
	for(i = 0;i < 8;i++) {
		data <<= 1;
		DREG |= SCL;
		if(DREG & SDA) data++;
		_wait();
		DREG &= ~SCL;
		_wait();
	}
	return data;
}

static void ack() {
	CREG |= (SCL + SDA);
	DREG &= ~SDA;
	DREG |= SCL;
	_wait();
	DREG &= ~SCL;
	CREG &= ~SDA;
}

static void nak() {
	CREG |= (SCL + SDA);
	DREG |= SDA;
	DREG |= SCL;
	_wait();
	DREG &= ~SCL;
	DREG &= ~SDA;
	CREG &= ~SDA;
}

static int status() {
	return DREG & SDA;	
}

static int send_ctrl(int ctrl) {
	int	ret, count;

	count = 0;
	do {
		start_i2c();
		ret = writebyte(ctrl);
		if(count++ > 1000) return -1;
	} while(ret == NAK);
	return 0;
}

static void i2c_init() {
	CREG |= SCL;
	DREG = 0;
}

#define RTC_WRITE	0xa2
#define RTC_READ	0xa3

static int rtc_read(int addr) {
	int	data, ret;

	ret = send_ctrl(RTC_WRITE);
	if(ret == -1) return -1;
	writebyte(addr);
	ret = send_ctrl(RTC_READ);
	if(ret == -1) return -1;
	data = readbyte();
	nak();
	stop_i2c();
	return data;
}

static int rtc_write(int addr, int data) {
	int	ret;

	ret = send_ctrl(RTC_WRITE);
	if(ret == -1) return -1;
	ret = writebyte(addr);
	ret = writebyte(data);
	stop_i2c();
	return 0;
}

static int get_value(int reg) {
	int	data;

	data = reg & 0x0f;
	if(reg & 0x10) data += 10;
	if(reg & 0x20) data += 20;
	if(reg & 0x40) data += 40;
	if(reg & 0x80) data += 80;
	return data;
}

static int set_value(int value) {
	int	reg, data;

	reg = (value / 10) << 4;
	data = value % 10;
	reg += data;
	return reg;
}
void send_b245(unsigned char data){
	while(PA.DR.BIT.B8){ ; } //TX?
	
	FT245 = data;
}

int main(int argc, char **argv) {
	int	year, month, week, day, hour, min, sec;
	int	i, value;
	int n;
	unsigned char c;

//	PAIORH |= 0x30; //PA21,PA20:out
	PFC.PAIORH.WORD |= 0x30; //PA21,PA20:out
	
	XPW = 0x3; //CPLD IO OUT
	
	//XPA = 0x55;
	//XPB = 0xaa;
	
	//for(;;);
	
	
	BSC.BCR1.BIT.A3SZ = 0; //CS3:8bitbus for PC/104
	
	DPW1 = 0x80; //T104-DIO72 All Out
	DPW2 = 0x80; //T104-DIO72 All Out
	DPW3 = 0x80; //T104-DIO72 All Out
	
	DPA1 = 0x1;
	
	for(i=0;i<10000;i++){
		PA.DR.WORD.H  |= 0x30; //PA21,PA20:H
		_wait();
		PA.DR.WORD.H  &= ~0x30; //PA21,PA20:H
		_wait();
	}


	


	i2c_init();
	if(argc == 3 && argv[1][0] == '-') {
		rtc_write(0, 0);
		sscanf(argv[2], "%d", &value);
		switch(argv[1][1]) {
		case 'Y':
			rtc_write(8, set_value(value));
			break;
		case 'M':
			rtc_write(7, set_value(value));
			break;
		case 'D':
			rtc_write(5, set_value(value));
			break;
		case 'h':
			rtc_write(4, set_value(value));
			break;
		case 'm':
			rtc_write(3, set_value(value));
			break;
		case 's':
			rtc_write(2, set_value(value));
			break;
		}
	} else {
		for(;;) {
			
			if(read(0, &c, 1) ){ break; }//L[͂Β~
			
			year = get_value(rtc_read(8));
			month = get_value(rtc_read(7) & 0x1f);
			week = get_value(rtc_read(6) & 0x07);
			day = get_value(rtc_read(5) & 0x3f);
			hour = get_value(rtc_read(4) & 0x3f);
			min = get_value(rtc_read(3) & 0x7f);
			sec = get_value(rtc_read(2) & 0x7f);
			printf("20%02d/%d/%d  ", year, month, day);
			printf("%d:%02d:%02d\r", hour, min, sec);
			//printf("%d:%02d:%02d", hour, min, sec);
			
		
			PA.DR.WORD.H = (~PA.DR.WORD.H & 0x30); //PA21,PA20:H
			
			
			if(!PA.DR.BIT.B9){//RX?
				send_b245(FT245);
			}
			//printf("PA:%04X\r", PA.DR.WORD.L);
			
			DPA1 <<= 1;
			DPB1 <<= 1;
			DPC1 <<= 1;
			
			DPA2 |= (DPA2 << 1);
			DPB2 |= (DPB2 << 1);
			DPC2 |= (DPC2 << 1);
			
			
			//DPA1 = (1 << i);
			//DPB1 = (1 << i);
			//DPC1 = (1 << i);
			
			
			XPA = (1 << i);
			XPB = (0x80 >> i);
			
			//PADRH &= ~0x30; //PA21,PA20:L

			if( ++i > 7){
			 	i = 0;
			 	//XPA = 0;
			 	//XPB = 0xff;
			   	DPA1 = 0x1;
			   	DPB1 = 0x1;
			   	DPC1 = 0x1;
		
			   	DPA2 = 0x1;
			   	DPB2 = 0x1;
			   	DPC2 = 0x1;
		
			 }
			
			
			sleep(100);
			
		}
	}
	
	

}
